$ontext
This GAMS code applies SCEQ to solve DSICE with climate tipping risk described
in the appendix of Cai and Judd (2023)

This code is based on
(1) Working paper about DICE-CJL model:
    Cai, Yongyang, Kenneth L. Judd and Thomas S. Lontzek (2012).
    Continuous-Time Methods for Integrated Assessment Models. NBER working paper No. w18365.
(2) DICE 2007 with 10 year time periods given in the webpage:
    http://nordhaus.econ.yale.edu/DICE2007_short.gms

* If using material from this code, the user should cite the following paper:
* Cai, Y., and K.L. Judd (2023). A Simple but Powerful Simulated Certainty Equivalent
*   Approximation Method for Dynamic Stochastic Problems. Quantitative Economics.
$offtext

parameter gamma elasticity of marginal Welfare of consumption / 1.45 /;
parameter Jbar tipping damage level / 0.05 /;

SETS  T                 Time periods                     / 1*601 / ;
alias(t, t1);

parameter nu hazard parameter /0.0035/
    TAthresh  threshold for potential tipping  / 1 /
    s    starting period
;
s = 1;


SCALARS

** Preferences
 beta    discount factor   / .985    /

** Population and technology
 POP0     2005 world population millions                  /6514     /
 GPOP0    Growth rate of population per year              /.035      /
 POPASYM  Asymptotic population                           / 8600    /
 A0       Initial level of total factor productivity      /.02722   /
 GA0      Initial growth rate for technology per year     /.0092      /
 DELA     Decline rate of technol change per year         /.001     /
 DK       Depreciation rate on capital per year           /.100     /
 GAMA     Capital elasticity in production function       /.300     /
 K0       2005 value capital trill 2005 US dollars        /137.     /

** Emissions
 SIG0     CO2-equivalent emissions-GNP ratio 2005         /.13418    /
 GSIGMA   Initial growth of sigma per year                /-.00730    /
 DSIG     Decline rate of decarbonization per year        /.003   /
 ELAND0   Carbon emissions from land 2005(GtC per year)   / 1.1000  /

** Carbon cycle
 MAT2000  Concentration in atmosphere 2005 (GtC)          /808.9   /
 MU2000   Concentration in upper strata 2005 (GtC)        /1255     /
 ML2000   Concentration in lower strata 2005 (GtC)        /18365    /
 b12      Carbon cycle transition matrix per year         /0.01908349 /
 b23      Carbon cycle transition matrix per year         /0.005405052     /

** Climate model
 T2XCO2   Equilibrium temp impact of CO2 doubling oC      / 3 /
 FEX0     Estimate of 2000 forcings of non-CO2 GHG        / -.06   /
 FEX1     Estimate of 2100 forcings of non-CO2 GHG        / 0.30   /
 TOCEAN0  2000 lower strat. temp change (C) from 1900     /.0068   /
 TATM0    2000 atmospheric temp change (C)from 1900       /.7307   /
 C1       Climate-equation coefficient for upper level per year    /.03713025    /
 C3       Transfer coeffic upper to lower stratum per year        /.2767396    /
 C4       Transfer coeffic for lower level per year                /.004801917    /
 FCO22X   Estimated forcings of equilibrium co2 doubling  /3.8     /

** Climate damage parameters calibrated for quadratic at 2.5 C for 2105
 A1       Damage intercept                                / 0.00000    /
 A2       Damage quadratic term                           /  0.0028388 /
* A3       Damage exponent                                 / 2.00       /

** Abatement cost
 EXPCOST2   Exponent of control cost function               /2.8   /
 PBACK      Cost of backstop 2005 000$ per tC 2005          /1.17  /
 BACKRAT    Ratio initial to final backstop cost            / 2    /
 GBACK      Initial cost decline backstop pc per year       /.005   /
 LIMMIU     Upper limit on control rate                     / 1    /

** Availability of fossil fuels
 FOSSLIM  Maximum cumulative extraction fossil fuels         / 6000  /
;

SET TLAST(T);
TLAST(T)  = YES$(ORD(T) EQ CARD(T));


PARAMETERS
  LAM           Climate model parameter
  L(T)          Level of population and labor
  AL(T)         Level of total factor productivity
  SIGMA(T)      CO2-equivalent-emissions output ratio
  FORCOTH(T)    Exogenous forcing for other greenhouse gases
  ETREE(T)      Emissions from deforestation
  cost1(t)      Adjusted cost for backstop
 ;

* Important parameters for the model
LAM     = FCO22X/ T2XCO2;
L(T) = popasym  - (popasym-POP0)*exp(-GPOP0*(ORD(T)-1));
AL(T) = A0*exp(GA0*(1-exp(-DELA*(ORD(T)-1)))/DELA);
SIGMA(T) = SIG0*exp(GSIGMA*(1-exp(-DSIG*(ORD(T)-1)))/DSIG);
FORCOTH(T)= FEX0+ 0.01*(FEX1-FEX0)*(ORD(T)-1)$((ORD(T)-1)<=100)+ 0.36$((ORD(T)-1)>100);
ETREE(T) = ELAND0*exp(-0.01*(ORD(T)-1));
cost1(T) = (PBACK*SIGMA(T)*(1+exp(-GBACK*(ORD(T)-1)))/EXPCOST2) * ((BACKRAT-1)/BACKRAT);

parameters
  b11          Carbon cycle transition matrix
  b21          Carbon cycle transition matrix
  b22          Carbon cycle transition matrix
  b32          Carbon cycle transition matrix
  b33          Carbon cycle transition matrix
;

b11 = 1 - b12;
b21 = 587.473*b12/1143.894;
b22 = 1 - b21 - b23;
b32 = 1143.894*b23/18340;
b33 = 1 - b32 ;



*****************************************************************




VARIABLES
 K(T)            Capital stock trillions US dollars
 MAT(T)          Carbon concentration in atmosphere GtC
 MU(T)           Carbon concentration in shallow oceans Gtc
 ML(T)           Carbon concentration in lower oceans GtC
 TATM(T)         Temperature of atmosphere in degrees C
 TOCEAN(T)       Temperature of lower oceans degrees C
 E(T)            CO2-equivalent emissions GtC
 C(T)            Consumption trillions US dollars
 MIU(T)          Emission control rate GHGs
 GrossY(T)
 Y(T)
 Probs(T)	     Probability that tipping event has not happened by the beginning of period t
 Welfare;

POSITIVE VARIABLES K, MAT, MU, ML, TATM, C, MIU, E;


*****************************************************************

EQUATIONS

 KK(T)            Capital balance equation before tipping
 KKtipped(t)      Capital balance equation after tipping
 MMAT(T)          Atmospheric concentration equation
 MMU(T)           Shallow ocean concentration
 MML(T)           Lower ocean concentration
 TATMEQ(T)        Temperature-climate equation for atmosphere
 TOCEANEQ(T)      Temperature-climate equation for lower oceans
 
 GrossOutputEq(t)
 NetOutputEq(t)
 
 NoTippingProbability1(T)	Probability that tipping event has not happened
 NoTippingProbability2(T)	Probability that tipping event has not happened
 
 ObjFun             Objective function
;

** Equations of the model

KK(T)$(ord(t)>=s and ord(t)<card(t))..          K(T+1)      =L= (1-DK)*K(T) + (1 - (1-Probs(T))*Jbar)*Y(T)  - C(T);
KKtipped(T)$(ord(t)>=s and ord(t)<card(t))..    K(T+1)      =L= (1-DK)*K(T) + (1 - Jbar)*Y(T)  - C(T);
MMAT(T)$(ord(t)>=s and ord(t)<card(t))..        MAT(T+1)    =E= MAT(T)*b11+MU(T)*b21 + SIGMA(T)*(1-MIU(T))*GrossY(t) + ETREE(T);
MML(T)$(ord(t)>=s and ord(t)<card(t))..         ML(T+1)     =E= ML(T)*b33+b23*MU(T);
MMU(T)$(ord(t)>=s and ord(t)<card(t))..         MU(T+1)     =E= MAT(T)*b12+MU(T)*b22+ML(T)*b32;
TATMEQ(T)$(ord(t)>=s and ord(t)<card(t))..      TATM(T+1)   =E= TATM(t)+C1*(FCO22X*(log(MAT(T)/596.4)/log(2))+FORCOTH(T) - LAM*TATM(t)-C3*(TATM(t)-TOCEAN(t)));
TOCEANEQ(T)$(ord(t)>=s and ord(t)<card(t))..    TOCEAN(T+1) =E= TOCEAN(T)+C4*(TATM(T)-TOCEAN(T));

GrossOutputEq(t)$(ord(t)>=s and ord(t)<card(t))..   GrossY(t) =e= AL(T)*L(T)**(1-GAMA)*K(T)**GAMA;
NetOutputEq(t)$(ord(t)>=s and ord(t)<card(t))..     Y(t) =e= ( 1 - cost1(T)*(MIU(T)**EXPcost2) )*GrossY(t) / (1+A1*TATM(T)+ A2*TATM(T)*TATM(T));

NoTippingProbability1(T)$(ord(t)>=s and ord(t)<card(t))..	Probs(T+1) =l= exp(-nu*(TATM(t)-TAthresh)) * Probs(T);
NoTippingProbability2(T)$(ord(t)>=s and ord(t)<card(t))..	Probs(T+1) =l= Probs(T);

ObjFun..             Welfare =E= sum( T$(ord(T)<card(T) and ord(t)>=s), (((C(T)/L(T))**(1-gamma))/(1-gamma)) * (beta**(ord(t)-s)*L(T)/POPASYM) );

*******************************
**  Upper and Lower Bounds

K.lo(T)         = 100;
MAT.lo(T)       = 10;
MU.lo(t)        = 100;
ML.lo(t)        = 1000;
TOCEAN.up(T)    = 20;
TOCEAN.lo(T)    = -1;
TATM.lo(t)      = 0;
TATM.up(t)      = 20;

Probs.l(T) = 1;

C.lo(T)         = 20;
miu.lo(t)       = 0.001;
miu.up(t)       = LIMMIU;


**********************************

** Solution options
option iterlim = 99900;
option reslim = 99999;
option solprint = off;
option limrow = 0;
option limcol = 0;
option nlp = conopt;


model CO2 / KK, MMAT, MMU, MML, TATMEQ, TOCEANEQ, GrossOutputEq, NetOutputEq, NoTippingProbability1, NoTippingProbability2, ObjFun /;

model CO2tipped / KKtipped, MMAT, MMU, MML, TATMEQ, TOCEANEQ, GrossOutputEq, NetOutputEq, ObjFun /;

 
set npath /1*102/;
parameter Nt   number of periods of interest;
Nt = card(npath)-2;
    
parameter
    Kpath(t,npath)
    MATpath(t,npath)
    MUpath(t,npath)
    MLpath(t,npath)    
    TApath(t,npath)
    TLpath(t,npath)
    miupath(t,npath)
    Cpath(t,npath)
    opt_SCC(t,npath)
    opt_tax(t,npath)
    lamK(t,npath)
    lamMAT(t,npath)
    lamMU(t,npath)
    lamML(t,npath)
    lamTA(t,npath)
    lamTL(t,npath)
    Welfares(t,npath)
;

Kpath(t,npath) = 1;
MATpath(t,npath) = 1;
MUpath(t,npath) = 0;
MLpath(t,npath) = 0;    
TApath(t,npath) = 0;
TLpath(t,npath) = 0;
miupath(t,npath) = 0;
Cpath(t,npath) = 0;
lamK(t,npath) = 0;
lamMAT(t,npath) = 0;
lamMU(t,npath) = 0;
lamML(t,npath) = 0;
lamTA(t,npath) = 0;
lamTL(t,npath) = 0;
Welfares(t,npath) = 0;
    

* solve the pre-tipping path
loop(npath$(ord(npath)=1),
    s = 1;
    
    K.fx('1') = K0;
    MAT.fx('1') = MAT2000;
    MU.fx('1') = MU2000;
    ML.fx('1') = ML2000;
    TATM.fx('1') = TATM0;
    TOCEAN.fx('1') = TOCEAN0;

* Probability that tipping event has not happened by the beginning of period t    
    Probs.fx('1') = 1;
    
    solve CO2 maximizing Welfare using nlp ;

    Kpath('1',npath) = K.l('1');
    MATpath('1',npath) = MAT.l('1');
    MUpath('1',npath) = MU.l('1');
    MLpath('1',npath) = ML.l('1');   
    TApath('1',npath) = TATM.l('1');
    TLpath('1',npath) = TOCEAN.l('1');
    miupath('1',npath) = miu.l('1');
    Cpath('1',npath) = C.l('1');
    lamK('1',npath) = KK.m('1');
    lamMAT('1',npath) = MMAT.m('1');
    lamMU('1',npath) = MMU.m('1');
    lamML('1',npath) = MML.m('1');
    lamTA('1',npath) = TATMEQ.m('1');
    lamTL('1',npath) = TOCEANEQ.m('1');

    loop(t1$(ord(t1)>1 and ord(t1)<=card(npath)),
* move one step forward
        s = ord(t1);

* fix the state variable at s conditional on that the tipping event has not happened
        K.fx(t1) = K.l(t1);
        MAT.fx(t1) = MAT.l(t1);
        MU.fx(t1) = MU.l(t1);
        ML.fx(t1) = ML.l(t1);
        TATM.fx(t1) = TATM.l(t1);
        TOCEAN.fx(t1) = TOCEAN.l(t1);
    
* Probability that tipping event has not happened by the beginning of period t
        Probs.fx(t1) = 1;
            
        solve CO2 maximizing Welfare using nlp ;
            
        Kpath(t1,npath) = K.l(t1);
        MATpath(t1,npath) = MAT.l(t1);
        MUpath(t1,npath) = MU.l(t1);
        MLpath(t1,npath) = ML.l(t1);   
        TApath(t1,npath) = TATM.l(t1);
        TLpath(t1,npath) = TOCEAN.l(t1);
        miupath(t1,npath) = miu.l(t1);
        Cpath(t1,npath) = C.l(t1);            
        lamK(t1,npath) = KK.m(t1);
        lamMAT(t1,npath) = MMAT.m(t1);
        lamMU(t1,npath) = MMU.m(t1);
        lamML(t1,npath) = MML.m(t1);
        lamTA(t1,npath) = TATMEQ.m(t1);
        lamTL(t1,npath) = TOCEANEQ.m(t1);
        Welfares(t1,npath) = Welfare.l;            
    );

* relax the fixed constraints on the state variables
    K.lo(t) = 0.01;
    K.up(t) = 1000000;
    MAT.lo(t) = 0;
    MAT.up(t) = 100000;
    MU.lo(t) = 0;
    MU.up(t) = 100000;
    ML.lo(t) = 0;
    ML.up(t) = 100000;
    TATM.lo(t) = -10;
    TATM.up(t) = 100;
    TOCEAN.lo(t) = -10;
    TOCEAN.up(t) = 100;
);    

parameter firstTip;
loop(t1,
    if(TApath(t1,'1') > TAthresh,
      firstTip = ord(t1)+1;
      break;
    );
);

* solve the tipped paths
loop(npath$(ord(npath)>=firstTip),
    s = ord(npath);

* fix the state variable at s conditional on that the tipping event has not happened
    K.fx(t1)$(ord(t1)=s) = Kpath(t1,'1');
    MAT.fx(t1)$(ord(t1)=s) = MATpath(t1,'1');
    MU.fx(t1)$(ord(t1)=s) = MUpath(t1,'1');
    ML.fx(t1)$(ord(t1)=s) = MLpath(t1,'1');
    TATM.fx(t1)$(ord(t1)=s) = TApath(t1,'1');
    TOCEAN.fx(t1)$(ord(t1)=s) = TLpath(t1,'1');
    
* if tipping event happended at period ord(npath)
    solve CO2tipped maximizing Welfare using nlp ;

    Kpath(t,npath)$(ord(t)>=s) = K.l(t);
    MATpath(t,npath)$(ord(t)>=s) = MAT.l(t);
    MUpath(t,npath)$(ord(t)>=s) = MU.l(t);
    MLpath(t,npath)$(ord(t)>=s) = ML.l(t);   
    TApath(t,npath)$(ord(t)>=s) = TATM.l(t);
    TLpath(t,npath)$(ord(t)>=s) = TOCEAN.l(t);
    miupath(t,npath)$(ord(t)>=s) = miu.l(t);
    Cpath(t,npath)$(ord(t)>=s) = C.l(t);            
    lamK(t,npath)$(ord(t)>=s) = KKtipped.m(t)/beta**(ord(t)-s);
    lamMAT(t,npath)$(ord(t)>=s) = MMAT.m(t)/beta**(ord(t)-s);
    lamMU(t,npath)$(ord(t)>=s) = MMU.m(t)/beta**(ord(t)-s);
    lamML(t,npath)$(ord(t)>=s) = MML.m(t)/beta**(ord(t)-s);
    lamTA(t,npath)$(ord(t)>=s) = TATMEQ.m(t)/beta**(ord(t)-s);
    lamTL(t,npath)$(ord(t)>=s) = TOCEANEQ.m(t)/beta**(ord(t)-s);
    Welfares(t,npath)$(ord(t)=s) = Welfare.l;

* relax the fixed constraints on the state variables
    K.lo(t) = 0.01;
    K.up(t) = 1000000;
    MAT.lo(t) = 0;
    MAT.up(t) = 100000;
    MU.lo(t) = 0;
    MU.up(t) = 100000;
    ML.lo(t) = 0;
    ML.up(t) = 100000;
    TATM.lo(t) = -10;
    TATM.up(t) = 100;
    TOCEAN.lo(t) = -10;
    TOCEAN.up(t) = 100;
);

* compute SCC and carbon tax
opt_SCC(t,npath) = -1*lamMAT(t,npath)*1000/(lamK(t,npath) + 0.000000000001);
opt_tax(t,npath) = expcost2*cost1(t)*miupath(t,npath)**(expcost2-1)/sigma(t)*1000;
opt_tax(t,npath)$(ord(npath)>1) = opt_tax(t,npath) * (1 - Jbar);

* compute Error errors
parameter netY(t,npath)
    Eind(t,nPath)
    dF(t,npath)
    dMitiCost(t,npath)
    Dam(t,npath)
    dDam(t,npath)
    
    probTipped(t)
    dprobTipped(t)
        
    errsK(t)
    errsMAT(t)
    errsMU(t)
    errsML(t)
    errsTA(t)
    errsTL(t)
;

Dam(t,npath)$(ord(t)<=card(npath)) = a1*TApath(t,npath) + a2*sqr(TApath(t,npath));
netY(t,npath)$(ord(t)<=card(npath)) = ( 1 - cost1(T)*(MIUpath(T,npath)**EXPcost2) )*
                (AL(T)*L(T)**(1-GAMA)*Kpath(T,npath)**GAMA) / (1+Dam(T,npath));
netY(t,npath)$(ord(npath)>1 and ord(t)>=ord(npath) and ord(t)<=card(npath)) = ( 1 - cost1(T)*(MIUpath(T,npath)**EXPcost2) )*
                (AL(T)*L(T)**(1-GAMA)*Kpath(T,npath)**GAMA) * (1 - Jbar) / (1+Dam(T,npath));
Eind(t,npath)$(ord(t)<=card(npath)) = SIGMA(T)*(1-MIUpath(T,npath))* (AL(T)*L(T)**(1-GAMA)*Kpath(T,npath)**GAMA);
dF(t,npath)$(ord(t)<=card(npath)) = FCO22X/(MATpath(t,npath)*log(2));
dMitiCost(t,npath)$(ord(t)<=card(npath)) = cost1(t)*(EXPcost2*(miupath(t,npath)**(EXPcost2-1)));
dDam(t,npath)$(ord(t)<=card(npath)) = a1 + 2*a2*TApath(t,npath);

probTipped(t)$(ord(t)<=card(npath)) = 1-exp(-nu*max(0, TApath(T,'1')-TAthresh));
dprobTipped(t)$(ord(t)<=card(npath)) = 0;
loop(t$(ord(t)<=card(npath)),
  if(TApath(t,'1') > TAthresh,
    dprobTipped(t) = nu*exp(-nu*(TApath(T,'1')-TAthresh));
  );
);

errsK(t)$(ord(t)<=Nt) = beta/lamK(t,'1') * [ (1-probTipped(t))*
    (lamK(t+1,'1')*((1-DK) + netY(t+1,'1')*GAMA / Kpath(t+1,'1')) + lamMAT(t+1,'1')*Eind(t+1,'1')*GAMA / Kpath(t+1,'1')) +
    probTipped(t)*sum( npath$(ord(npath)=ord(t)+1), 
    (lamK(t+1,npath)*((1-DK) + netY(t+1,npath)*GAMA / Kpath(t+1,npath)) + lamMAT(t+1,npath) * Eind(t+1,npath)*GAMA / Kpath(t+1,npath)) ) ] - 1;
errsMAT(t)$(ord(t)<=Nt) = beta/lamMAT(t,'1') * [ (1-probTipped(t))*
    (lamMAT(t+1,'1')*b11 + lamMU(t+1,'1')*b12 + lamTA(t+1,'1')*C1*dF(t+1,'1')) + 
    probTipped(t)*sum( npath$(ord(npath)=ord(t)+1),
    (lamMAT(t+1,npath)*b11 + lamMU(t+1,npath)*b12 + lamTA(t+1,npath)*C1*dF(t+1,npath)) ) ] - 1;  
errsMU(t)$(ord(t)<=Nt) = beta/lamMU(t,'1') * [ (1-probTipped(t))*
    (lamMAT(t+1,'1')*b21 + lamMU(t+1,'1')*b22 + lamML(t+1,'1')*b23) +
    probTipped(t)*sum( npath$(ord(npath)=ord(t)+1),
    (lamMAT(t+1,npath)*b21 + lamMU(t+1,npath)*b22 + lamML(t+1,npath)*b23) ) ] - 1;
errsML(t)$(ord(t)<=Nt) = beta/lamML(t,'1') * [ (1-probTipped(t))*
    (lamMU(t+1,'1')*b32 + lamML(t+1,'1')*b33) + 
    probTipped(t)*sum( npath$(ord(npath)=ord(t)+1),
    (lamMU(t+1,npath)*b32 + lamML(t+1,npath)*b33) ) ] - 1;
errsTA(t)$(ord(t)<=Nt) = beta/lamTA(t,'1') * [ (1-probTipped(t))*
    (lamTA(t+1,'1')*(1-C1*LAM-C1*C3) + lamTL(t+1,'1')*C4 + lamK(t+1,'1')*dDam(t+1,'1')*((-netY(t+1,'1'))/(1+Dam(t+1,'1')))) +
    probTipped(t)*sum( npath$(ord(npath)=ord(t)+1),
    (lamTA(t+1,npath)*(1-C1*LAM-C1*C3) + lamTL(t+1,npath)*C4 + lamK(t+1,npath)*dDam(t+1,npath)*((-netY(t+1,npath))/(1+Dam(t+1,npath)))) ) + 
    beta*(1-probTipped(t))*[ (dprobTipped(t+1))*sum(npath$(ord(npath)=ord(t)+2), Welfares(t+2,npath)) +
        (-dprobTipped(t+1)) * Welfares(t+2,'1') ] ] - 1;
errsTL(t)$(ord(t)<=Nt) = beta/lamTL(t,'1') * [ (1-probTipped(t))*    
    (lamTA(t+1,'1')*C1*C3 + lamTL(t+1,'1')*(1-C4)) + 
    probTipped(t)*sum( npath$(ord(npath)=ord(t)+1),
    (lamTA(t+1,npath)*C1*C3 + lamTL(t+1,npath)*(1-C4)) ) ] - 1;
    
* save the different paths for simulation by other software like Matlab
file SCEQ_DSICE_Tip_sol;
put SCEQ_DSICE_Tip_sol;
SCEQ_DSICE_Tip_sol.nw = 12;
SCEQ_DSICE_Tip_sol.nr = 2;
SCEQ_DSICE_Tip_sol.nz = 1e-15;
loop(npath,
  loop(t$(ord(t)<=Nt),
    put t.tl:4:0;
    put Kpath(t,npath):13:5;
    put MATpath(t,npath):13:5;
    put MUpath(t,npath):13:5;
    put MLpath(t,npath):13:5;
    put TApath(t,npath):13:5;
    put TLpath(t,npath):13:5;
    put miupath(t,npath):13:5;
    put Cpath(t,npath):13:5;
    put opt_SCC(t,npath):13:5;
    put opt_tax(t,npath):13:5;        
    put /;
  );
);

file SCEQ_DSICE_Tip_errs;
put SCEQ_DSICE_Tip_errs;
SCEQ_DSICE_Tip_errs.nw = 12;
SCEQ_DSICE_Tip_errs.nr = 2;
SCEQ_DSICE_Tip_errs.nz = 1e-15;
loop(t$(ord(t)<=Nt),
    put t.tl:4:0;
    put errsK(t):13:5;
    put errsMAT(t):13:5;
    put errsMU(t):13:5;
    put errsML(t):13:5;
    put errsTA(t):13:5;
    put errsTL(t):13:5;
    put /;
);
