function [err,runtime] = SCEQ_newKeynesian()

% This code solves the New Keynesian model with zero lower bound 
%
% If using material from this code, the user should cite the following paper:
% Cai, Y., and K.L. Judd (2023). A Simple but Powerful Simulated Certainty Equivalent
%   Approximation Method for Dynamic Stochastic Problems. Quantitative Economics.
%----------------------------------------------------------------------

% number of periods for one optimization problem (i.e., Delta_s)
T = 50;

% time of interest
Tstar = 20;

% parameter values
eta = 1; % labor supply elasticity
rho = 0.8; % persistence of discount rate shock
sigma = 0.005; % standard deviation of discount rate shock 
alpha = 6; % elasticity of substitution across intermediate goods
theta = 0.9; % Calvo parameter
sg = 0.2; % ratio of govenment spending to output 
phi_pi = 2.5; % response to inflation (monetary policy rule) 
phi_y = 0.25; % response to output (monetary policy rule)
pi_ss = 1.005; % steady state inflation 
beta0 = 0.994; % initial value of beta
betass = 0.994; % steady state beta

% compute values at the steady state
rss = pi_ss/betass-1;
chi2ss = 1/((1-sg)*(1-theta*betass*pi_ss^(alpha-1)));
qss = ((1-theta*pi_ss^(alpha-1))/(1-theta))^(1/(1-alpha));
chi1ss = chi2ss*qss*(alpha-1)/alpha;
vss = (1-theta)*qss^(-alpha) / (1-theta*pi_ss^alpha);
yss = (chi1ss*(1-theta*betass*pi_ss^alpha)/(vss^eta))^(1/(1+eta));
zss = rss;

% initial value of v
v0 = vss;

options = optimoptions('fsolve','Display','off');

% number of simulation paths
nsim = 1000;

betas = zeros(Tstar+1,nsim);
vsols = zeros(Tstar+1,nsim);
chi1sols = zeros(Tstar,nsim);
chi2sols = zeros(Tstar,nsim);
ysols = zeros(Tstar,nsim);
pisols = zeros(Tstar,nsim);
zsols = zeros(Tstar,nsim);

t0 = datetime('now');

parfor i = 1:nsim
    beta1 = zeros(Tstar+1,1);
    vsol1 = zeros(Tstar+1,1);
    chi1sol1 = zeros(Tstar,1);
    chi2sol1 = zeros(Tstar,1);
    ysol1 = zeros(Tstar,1);
    pisol1 = zeros(Tstar,1);
    zsol1 = zeros(Tstar,1);
    
% starting at the initial state    
    beta1(1) = beta0;
    vsol1(1) = v0;    
    
% variables (initial guess)
    xsol = zeros(7*T+7,1);
    xsol(1:T+1) = vss;
    xsol(T+2:2*T+2) = chi1ss;
    xsol(2*T+3:3*T+3) = chi2ss;
    xsol(3*T+4:4*T+4) = yss;
    xsol(4*T+5:5*T+5) = pi_ss;
    xsol(5*T+6:6*T+6) = qss;
    xsol(6*T+7:7*T+7) = zss;
  
% Initialization step for simulating shocks    
% set the seed for the MATLAB random number generator as i (the results
% can be replicated with the same seed)
    rng(i);    
    shocks = randn(Tstar+1,1);
    
    for s = 1:Tstar        
        % use certainty equivalent approximation  
        beta = zeros(T+1,1);
        beta(1) = beta1(s);
        for j=1:T
            beta(j+1) = exp((1-rho)*log(betass) + rho*log(beta(j)));
        end
        
        % optimization step
        x0 = xsol;
        xsol = fsolve(@(x) newKeynesian_equiFun(x,T,vsol1(s),beta,theta,...
            alpha,rss,pi_ss,phi_pi,yss,phi_y,eta,sg,chi1ss,chi2ss,vss,qss,zss),x0,options);
        
        % save solutions
        chi1sol1(s) = xsol(T+2);
        chi2sol1(s) = xsol(2*T+3);
        ysol1(s) = xsol(3*T+4);
        pisol1(s) = xsol(4*T+5);
        zsol1(s) = xsol(6*T+7);
        
        % simulation step
        vsol1(s+1) = xsol(2);
        beta1(s+1) = exp((1-rho)*log(betass) + rho*log(beta1(s)) + sigma*shocks(s));
    end
    
    chi1sols(:,i) = chi1sol1;
    chi2sols(:,i) = chi2sol1;
    ysols(:,i) = ysol1;
    pisols(:,i) = pisol1;
    zsols(:,i) = zsol1;
    
    vsols(:,i) = vsol1;
    betas(:,i) = beta1;    
end

t1 = datetime('now');
runtime = minutes(t1-t0);

% Euler error at the first period
err1 = abs(1 - mean(betas(2,:).*(1+max(0,zsols(1,:)))./pisols(2,:).*ysols(1,:)./ysols(2,:)));
err2 = abs(1 - mean((ysols(1,:).^(1+eta).*vsols(2,:).^eta + theta* ...
        (betas(2,:).*pisols(2,:).^alpha .* chi1sols(2,:)))./chi1sols(1,:)));
err3 = abs(1 - mean((1/(1-sg) + theta* ...
        (betas(2,:).*pisols(2,:).^(alpha-1) .* chi2sols(2,:)))./chi2sols(1,:)));
err = max([err1 err2 err3]);

% close the parallel pool
delete(gcp('nocreate'))

end


function ceq=newKeynesian_equiFun(x,T,v0,beta,theta,alpha,rss,pi_ss,...
    phi_pi,yss,phi_y,eta,sg,chi1ss,chi2ss,vss,qss,zss)
% variables: v(1),v(2),...,v(T+1), chi1(1),chi1(2),...,chi(T+1), 
% chi2(1),chi2(2),...,chi2(T+1), y(1),y(2),...,y(T+1), 
% pi(1),pi(2),...,pi(T+1), q(1),q(2),...,q(T+1), 
% z(1),z(2),...,z(T+1)

v = x(1:T+1);
chi1 = x(T+2:2*T+2);
chi2 = x(2*T+3:3*T+3);
y = x(3*T+4:4*T+4);
pi_t = x(4*T+5:5*T+5);
q = x(5*T+6:6*T+6);
z = x(6*T+7:7*T+7);

ceq = zeros(7*T+7,1);

% initial v
ceq(1) = v(1) - v0;

% transition of v
ceq(2:T+1) = (1-theta).*(q(1:T).^(-alpha)) + theta.*(pi_t(1:T).^alpha).*v(1:T) - v(2:T+1);

% national interest
ceq(T+2:2*T+1) = (1+rss).*(pi_t(1:T)./pi_ss).^phi_pi .* (y(1:T)./yss).^phi_y - 1 - z(1:T);

% q fun
ceq(2*T+2:3*T+1) = q(1:T).^(alpha-1) .* (1-theta.*(pi_t(1:T).^(alpha-1)))./(1-theta) - 1;

% chi1 fun
ceq(3*T+2:4*T+1) = (y(1:T).^(1+eta)).*(v(2:T+1).^eta) + ...
    theta.*beta(2:T+1).*(pi_t(2:T+1).^alpha).*chi1(2:T+1) - chi1(1:T);

% chi2 fun
ceq(4*T+2:5*T+1) = 1/(1-sg) + theta*beta(2:T+1).*(pi_t(2:T+1).^(alpha-1)).*chi2(2:T+1) - chi2(1:T);

% chi1 vs chi2 
ceq(5*T+2:6*T+1) = chi1(1:T) - q(1:T).*chi2(1:T).*((alpha-1)/alpha);

% Euler eq
ceq(6*T+2:7*T+1) = pi_t(2:T+1).*y(2:T+1) - beta(2:T+1).*(1+max(0,z(1:T))).*y(1:T);

ceq(7*T+2) = chi1(T+1) - chi1ss;
ceq(7*T+3) = chi2(T+1) - chi2ss;
ceq(7*T+4) = y(T+1) - yss;
ceq(7*T+5) = v(T+1) - vss;
ceq(7*T+6) = q(T+1) - qss;
ceq(7*T+7) = z(T+1) - zss;

end


