** This code applies SCEQ to solve a multi-country real business cycle problem
** described on the appendix of Cai and Judd (2023)
*
* If using material from this code, the user should cite the following paper:
* Cai, Y., and K.L. Judd (2023). A Simple but Powerful Simulated Certainty Equivalent
*   Approximation Method for Dynamic Stochastic Problems. Quantitative Economics.
*----------------------------------------------------------------------

$include RBC_SCEQ_common.gms

*********************
* run SCEQ

parameter 
    Cpath(j,tt,nsim) simulated consumption paths
    Kpath(j,tt,nsim) simulated capital paths
    thetapath(j,tt,nsim) simulated paths of shocks
    systemShock     realized system shock     
;

set niter / 1*10 /;

* Initialization step for simulating shocks
thetapath(j,'1',nsim) = 1;
loop((tt,nsim)$(ord(tt)<card(tt)),
  systemShock = sigma*normal(0,1);
  thetapath(j,tt+1,nsim) = thetapath(j,tt,nsim)**rho * exp(sigma*normal(0,1)+systemShock);
);

Kpath(j,'1',nsim) = K0(j);

* solve the first period only: note that its solution is independent of simulation
* starting period s
s = 1;    

* optimization step 

* fix the state variable at s 
K.fx(j,'1') = K0(j);
        
theta(j,t)$(ord(t)>=s) = thetapath(j,'1','1')**(rho**(ord(t)-s));

loop(niter,
    solve busc using nlp maximizing obj;
    if((busc.MODELSTAT<=2 and busc.SOLVESTAT=1),
        break;
    );
);
abort$(busc.MODELSTAT>2 or busc.SOLVESTAT<>1) "FAILED in solving!";

Cpath(j,'1',nsim) = C.l(j,'1');
  
* simulation step
Kpath(j,'2',nsim) = K.l(j,'2');
  
* iterate over periods of interest
loop(tt$(ord(tt)>1),
* starting period s
  s = ord(tt);    

  loop(nsim,  
* fix the state variable at s 
    K.fx(j,tt) = Kpath(j,tt,nsim);
        
    theta(j,t)$(ord(t)>=s) = thetapath(j,tt,nsim)**(rho**(ord(t)-s));

* optimization step 
    loop(niter,
        solve busc using nlp maximizing obj;
        if((busc.MODELSTAT<=2 and busc.SOLVESTAT=1),
            break;
        );
    );
    abort$(busc.MODELSTAT>2 or busc.SOLVESTAT<>1) "FAILED in solving!";

    Cpath(j,tt,nsim) = C.l(j,tt);
    
* simulation step
    Kpath(j,tt+1,nsim) = K.l(j,tt+1);
  );
);

***************
* error checking for the first period 
parameter errs(j);

errs(j) = abs( 1 - beta/card(nsim)*sum( nsim, ( 1-delta +
    thetapath(j,'2',nsim)*A*alpha*(Kpath(j,'2',nsim)**(alpha-1)) )*Cpath(j,'1',nsim)/Cpath(j,'2',nsim) ) );

display errs;

***************
* Output solutions 

File sol_SCEQ_RBC_theta /sol_SCEQ_RBC_theta.csv/;
sol_SCEQ_RBC_theta.pc=5;
sol_SCEQ_RBC_theta.pw=4000;

Put sol_SCEQ_RBC_theta;

loop(nsim,
  loop(tt,
    put tt.tl::4;    
    loop(j,
      put thetapath(j,tt,nsim)::6;
    );
    put /;
  );
);

File sol_SCEQ_RBC_C /sol_SCEQ_RBC_C.csv/;
sol_SCEQ_RBC_C.pc=5;
sol_SCEQ_RBC_C.pw=4000;

Put sol_SCEQ_RBC_C;

loop(nsim,
  loop(tt,
    put tt.tl::4;    
    loop(j,
      put Cpath(j,tt,nsim)::6;
    );
    put /;
  );
);


File sol_SCEQ_RBC_K /sol_SCEQ_RBC_K.csv/;
sol_SCEQ_RBC_K.pc=5;
sol_SCEQ_RBC_K.pw=4000;

Put sol_SCEQ_RBC_K;

loop(nsim,
  loop(tt,
    put tt.tl::4;
    loop(j,
      put Kpath(j,tt,nsim)::6;
    );    
    put /;
  );
);

* display the running time in minutes
scalar elapsed;
elapsed = (jnow - starttime)*24*60;
display elapsed;

