** This code applies SCEQ to solve a multi-country real business cycle problem
** described in the appendix of Cai and Judd (2023)
*
* If using material from this code, the user should cite the following paper:
* Cai, Y., and K.L. Judd (2023). A Simple but Powerful Simulated Certainty Equivalent
*   Approximation Method for Dynamic Stochastic Problems. Quantitative Economics.
*----------------------------------------------------------------------

scalar starttime;
starttime = jnow;

parameter seedv
    ncountry;
    
option seed = %seedv%;

set j countries /1*%ncountry%/;
set t time /1*71/;
set tt(t) periods of interest /1*20/;
set nsim /1*167/;

parameters
beta        discount rate /0.99/
alpha       capital cost share /0.36/
delta       capital stock depreciation /0.025/
rho         persistence parameter for TFP growth /0.95/
sigma       standard deviation of the stochastic productivity level / 0.01 /
A           technology parameter
s           starting period
DT          number of periods for SCEQ / 50 /
tau(j)      weight
k0(j)       initial capital
kmin        smallest capital    / 0.8 /
kmax        largest capital     / 1.2 /
;

A = (1 - (1-delta)*beta) / (alpha * beta);

tau(j) = 1;
k0(j) = exp(log(kmin) + (log(kmax)-log(kmin))*(ord(j)-1)/(card(j)-1));

display A, k0;

parameter
    theta(j,t)    productivity shock    
;


*************
* define model

Variables
obj objective criterion
;

Positive variables
k(j,t) capital stock
c(j,t) consumption
;

Equations
objfun Objective function
BudgetConstraint(t) budget constraint
;

objfun .. 
obj =e= sum(j, tau(j) * sum(t$(ord(t)>=s and ord(t)<s+DT), beta**(ord(t)-s)*log(c(j,t)))) + 
  sum(j, tau(j) * sum(t$(ord(t)=s+DT), beta**(ord(t)-s)*log(0.75*A*(k(j,t)**alpha))/(1-beta))) ;

BudgetConstraint(t)$(ord(t)>=s and ord(t)<s+DT) .. 
sum(j, c(j,t) + k(j,t+1)) =e= sum(j, theta(j,t)*A*(k(j,t)**alpha) + (1-delta)*k(j,t));

* Bound Constraints
k.lo(j,t) = 0.001;
k.up(j,t) = 1000;
c.lo(j,t) = 0.001;
c.up(j,t) = 1000;

* Initial Guess
s = 1;
k.l(j,t) = 1;
c.l(j,t) = A-delta;
obj.l = sum(j, tau(j) * sum(t$(ord(t)>=s and ord(t)<s+DT), beta**(ord(t)-s)*log(c.l(j,t)))) +
  sum(j, tau(j) * sum(t$(ord(t)=s+DT), beta**(ord(t)-s)*log(0.75*A*(k.l(j,t)**alpha))/(1-beta)));


options limrow = 0, limcol = 0;
option reslim = 10000;
option iterlim = 10000;
option solprint = off;
option nlp = conopt;

model busc /all/;


